﻿#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QTableWidget>


#include <QApplication>
#include <QHeaderView>
#include <QStandardItemModel>
#include <QTableView>
#include "spinboxdelegate.h"
#include <QFile>
#include <QFileDialog>


QColor GraphColors[] = {Qt::black,Qt::blue,Qt::green, Qt::magenta, Qt::darkCyan, Qt::darkGray};
int GraphColorsCount = sizeof(GraphColors)/sizeof(GraphColors[0]);

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    setGeometry(400, 250, 542, 390);

    IsRunning = false;

    customPlot = ui->customPlot;

    customPlot->legend->setVisible(false);
    customPlot->legend->setFont(QFont("Helvetica",9));
    customPlot->setLocale(QLocale(QLocale::German, QLocale::Germany));

    customPlot->xAxis->setTickLabelRotation(0);
    customPlot->xAxis->setAutoTickCount(9);
    customPlot->xAxis->setNumberFormat("f");
    customPlot->xAxis->setNumberPrecision(0);
    customPlot->xAxis->setLabel("Zeit [s]");

    customPlot->yAxis->setLabel("Temperatur [°C]");
    customPlot->yAxis->setRangeLower(0);
    customPlot->yAxis->setRangeUpper(270);
    customPlot->xAxis->setRangeLower(0);
    customPlot->xAxis->setRangeUpper(300);

    //customPlot->xAxis->settickLabels()


    DesiredValueGraph = customPlot->addGraph();
    QPen pen;
    pen.setStyle(Qt::DashLine);
    pen.setWidth(2);
    pen.setColor(Qt::red);
    DesiredValueGraph->setName("Solltemperatur");
    DesiredValueGraph->setPen(pen);

    customPlot->axisRect()->setupFullAxesBox();

    ConfigUI();

    QStringList HeadersList;
    HeadersList <<"Temp [°C]"<<"Time [s]";

    TempCurveModel = new QStandardItemModel(10, 2);
    TempCurveModel->setHorizontalHeaderLabels(HeadersList);


    ui->tableView->setModel(TempCurveModel);
    SpinBoxDelegate * delegate = new SpinBoxDelegate;
    ui->tableView->setItemDelegate(delegate);
    ui->tableView->horizontalHeader()->setStretchLastSection(true);
    for (int row = 0; row < TempCurveModel->rowCount(); ++row) {
        for (int column = 0; column < TempCurveModel->columnCount(); ++column) {
            QModelIndex index = TempCurveModel->index(row, column);
            TempCurveModel->setData(index, QVariant(0));
        }
    }

    ui->tableView->setColumnWidth(0,70);
    ui->tableView->setColumnWidth(1,70);
    ui->tableView->setMaximumWidth(180);
    ui->tableView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    ui->tableView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    ui->ServoLCD->display(ui->ServoSlider->value());

    ui->HeaterLED->setColor(Qt::red);

    ui->ConnectionLED->setColor(Qt::red);
    connect(&ComModul,SIGNAL(LineReceived(QString)),this,SLOT(ParseReceivedLine(QString)));
    connect(&Tick1000msTimer,SIGNAL(timeout()),this,SLOT(Tick1000ms()));
    Tick1000msTimer.setSingleShot(false);
    Tick1000msTimer.start(1000);

    TempChanCheckBoxChanged();

    connect(TempCurveModel,SIGNAL(itemChanged(QStandardItem*)),this, SLOT(TempCurveItemChanged(QStandardItem*)));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::Tick1000ms()
{
    if(!ComModul.isOpen()){
        ui->ConnectionLED->setColor(Qt::red);
        ComModul.ConnectToSerialDevice(0x16C0,0x047A);
    }else{
        ComModul.Flush();
        ComModul.SendLine(QString("AC=%1").arg(ui->HeaterLCD->value()));
        ComModul.SendLine("alltemps");//leerer Befehl um Gerät in definierten zustand zu bringen;
        QApplication::processEvents();
        ui->ConnectionLED->setColor(Qt::green);
    }

    ui->HeaterLED->turnOn(ui->HeaterLCD->value() != 0);
}



void MainWindow::TempCurveItemChanged(QStandardItem *item)
{
    Q_UNUSED(item)
    qDebug()<<"Temp Item Changed";

    TempCurveSanityCheck();

    QVector<double>Time, Temp;

    for (int row = 0; row < TempCurveModel->rowCount(); row++) {
        Temp.append(TempCurveModel->data(TempCurveModel->index(row, 0)).toDouble());
        Time.append(TempCurveModel->data(TempCurveModel->index(row, 1)).toDouble());
    }
    DesiredValueGraph->setData(Time,Temp);

    int MaxTime = 0;   
    foreach (double T, Time) {
        if((int)T > MaxTime){
            MaxTime = T;
        }
    }
    customPlot->xAxis->setRangeUpper( (((MaxTime-1)/100)+1)*100);
    customPlot->replot();
}

void MainWindow::TempChanCheckBoxChanged()
{
    for(int i = 0; i < TEMP_CHAN_COUNT; i++){
        TempChannels[i].Graph->setVisible(TempChannels[i].CheckBoxShow->isChecked());
    }
    customPlot->replot();
}




void MainWindow::ConfigUI()
{

    for(int i = 0; i < TEMP_CHAN_COUNT; i++){
        TempChannels[i].LCD = static_cast<QLCDNumber*>(ui->ChannelsGridLayout->itemAtPosition(0,i)->widget());
        TempChannels[i].CheckBoxShow = static_cast<QCheckBox*>(ui->ChannelsGridLayout->itemAtPosition(1,i)->widget());
        TempChannels[i].Graph = customPlot->addGraph();


        QPen pen;
        pen.setColor(GraphColors[i]);
        TempChannels[i].Graph->setName(QString("T%1").arg(i+1));
        TempChannels[i].Graph->setPen(pen);

        QPalette Pal = TempChannels[i].LCD->palette();
        Pal.setColor(QPalette::Normal, QPalette::WindowText, GraphColors[i]);
        Pal.setColor(QPalette::Inactive, QPalette::WindowText, GraphColors[i]);
        TempChannels[i].LCD->setPalette(Pal);

        TempChannels[i].Values.clear();

        connect(TempChannels[i].CheckBoxShow,SIGNAL(clicked()),this,SLOT(TempChanCheckBoxChanged()));
    }
}

void MainWindow::ParseReceivedLine(QString ReceivedLine)
{
    if(ReceivedLine.contains("alltemps")){//am Anfang der Temp-Übertagrung
        if(IsRunning){
            if(TimeAxisPoints.last() >= customPlot->xAxis->range().upper){
                on_ButtonStart_clicked();//Stoppen
            }else{
                TimeAxisPoints.append(TimeAxisPoints.last()+1);
            }
        }
    }else if(ReceivedLine.at(0) == 'T' && ReceivedLine.at(2) == '='){
        unsigned int Index =  ReceivedLine.at(1).digitValue()-1;
        if(Index < TEMP_CHAN_COUNT){
            ReceivedLine.remove(0,3);
            ReceivedLine.replace(" ","");
            ReceivedLine.replace(",",".");

            if(Index == TEMP_CHAN_COUNT-1){//am Ende der Temp-Übertagrung

                double CurrentValue = (TempChannels[1].LCD->value() + TempChannels[2].LCD->value())/2;
                QString TempStr;
                TempStr.setNum(CurrentValue,'f',1);
                ui->TempNumberIst->display(TempStr);

                if(IsRunning){
                    if(!TimeAxisPoints.isEmpty()){
                        double DesiredValue = GetDesiredValue(TimeAxisPoints.last());
                        if(DesiredValue < 0){
                            on_ButtonStart_clicked();//Stoppen
                        }else{
                            TempStr.setNum(DesiredValue,'f',1);
                            ui->TempNumberSoll->display(TempStr);

                            int Delta = DesiredValue-CurrentValue;

                            double Ramp = GetCurrentRamp(TimeAxisPoints.last());
                            int PlusDueToRamp = Ramp*8;
                            qDebug()<<"Ramp: "<<Ramp<<"K/s";

                            int PlusDueToEndTemp =  DesiredValue/60;

                            int HeaterSetting = 0;
                            if(DesiredValue  <= CurrentValue){
                                HeaterSetting = Delta + PlusDueToEndTemp + PlusDueToRamp-1;
                            }else{
                                if(PlusDueToRamp < 0){PlusDueToRamp = 0;}
                                HeaterSetting = Delta/10 + PlusDueToEndTemp + PlusDueToRamp +1;
                            }
                            if(HeaterSetting > 8){HeaterSetting = 8;}
                            if(HeaterSetting < 0){HeaterSetting = 0;}
                            ui->HeaterSlider->setValue(HeaterSetting);

                            if(HeaterSetting == 0){
                                if(Delta <= -5){SetDoorOpen(2);}
                                else if(Delta <= -3){SetDoorOpen(1);}
                                else{SetDoorOpen(0);}
                            }else{
                                SetDoorOpen(0);
                            }
                        }
                    }
                }else{
                     ui->TempNumberSoll->display("--");
                }
            }
            int StarPos = ReceivedLine.indexOf("*");
            if(StarPos >0){ReceivedLine.truncate(StarPos);}

            TempChannels[Index].LCD->display(ReceivedLine);
            if(IsRunning){
                bool ok;
                TempChannels[Index].Values.append(ReceivedLine.toDouble(&ok));
                if(!ok){
                    TempChannels[Index].Values.last() = -300;
                }
                TempChannels[Index].Graph->setData(TimeAxisPoints,TempChannels[Index].Values);
                if(Index == TEMP_CHAN_COUNT-1){
                   customPlot->replot();
                }
            }
        }
    }
}


void MainWindow::on_ButtonSaveProfile_clicked()
{
    QString SaveFileName = QFileDialog::getSaveFileName(this, "Speichere", QDir::currentPath(),"Temperaturprofil (*.tmp)");
    if(SaveFileName.isEmpty()){
        return;
    }
    QFile SaveFile(SaveFileName);
    if(!SaveFile.open(QIODevice::WriteOnly  | QIODevice::Text)){
        return;
    }
    QTextStream stream(&SaveFile);

    stream<<QString("Temp;Time\n");
    for(int i = 0; i < 10; i++){
        double Temp = TempCurveModel->data(TempCurveModel->index(i, 0)).toDouble();
        double Time = TempCurveModel->data(TempCurveModel->index(i, 1)).toDouble();

        stream<<QString("%1;%2\n").arg(Temp).arg(Time);
    }
    SaveFile.close();
}

void MainWindow::on_ButtonLoadProfile_clicked()
{
    QString LoadFileName = QFileDialog::getOpenFileName(this, "Lade", QDir::currentPath(),"Temperaturprofil (*.tmp)");
    if(LoadFileName.isEmpty()){
        return;
    }

    QFile LoadFile(LoadFileName);
    if(!LoadFile.open(QIODevice::ReadOnly)){
        return;
    }
    QString FileContent = LoadFile.readAll();

    QStringList Lines = FileContent.split(QRegExp("[\r\n]"),QString::SkipEmptyParts);
    for(int i = 1; i < TempCurveModel->rowCount(); i++){
        bool FoundValues = false;

        if(i < Lines.size()){
            QStringList Fields = Lines.at(i).split(';',QString::SkipEmptyParts);
            if(Fields.size() >= 2){
                TempCurveModel->setData(TempCurveModel->index(i-1, 0),Fields.at(0).toInt());
                TempCurveModel->setData(TempCurveModel->index(i-1, 1),Fields.at(1).toInt());
                FoundValues = true;
            }
        }
        if(!FoundValues){
            TempCurveModel->setData(TempCurveModel->index(i-1, 0),0);
            TempCurveModel->setData(TempCurveModel->index(i-1, 1),0);
        }
    }
}

void MainWindow::on_ButtonStart_clicked()
{
    if(IsRunning == false){
        TimeAxisPoints.clear();
        TimeAxisPoints.append(0);
        for(int i = 0; i < TEMP_CHAN_COUNT; i++){
            TempChannels[i].Values.clear();
            TempChannels[i].Values.append(TempChannels[i].LCD->value());
            TempChannels[i].Graph->setData(TimeAxisPoints,TempChannels[i].Values);
        }
        IsRunning = true;
        ui->ButtonStart->setText("Stop");
    }else{
        IsRunning = false;
        ui->HeaterSlider->setValue(0);
        ui->ButtonStart->setText("Start");        
    }
}

void MainWindow::on_ButtonCSVexport_clicked()
{
    QString SaveFileName = QFileDialog::getSaveFileName(this, "Export CSV", QDir::currentPath(),"CSV (*.csv)");
    if(SaveFileName.isEmpty()){
        return;
    }
    QFile SaveFile(SaveFileName);
    if(!SaveFile.open(QIODevice::WriteOnly  | QIODevice::Text)){
        return;
    }
    QTextStream stream(&SaveFile);

    stream.setLocale(QLocale::German);

    stream<<QString("Time");
    for(int i = 0; i < TEMP_CHAN_COUNT; i++){
      stream<<";Temp "<<i;
    }
    stream<<"\n";

    for(int k = 0; k < TimeAxisPoints.size(); k++){
        stream<<TimeAxisPoints.value(k);

        for(int i = 0; i < TEMP_CHAN_COUNT; i++){
            stream<<";"<<TempChannels[i].Values.value(k);

        }
        stream<<"\n";
    }
    SaveFile.close();
}

void MainWindow::TempCurveSanityCheck()
{
    QBrush red;
    red.setColor(Qt::red);

    QBrush black;
    black.setColor(Qt::black);


    QVector<double>Time, Temp;

    for (int row = 0; row < TempCurveModel->rowCount(); row++) {
        Temp.append(TempCurveModel->data(TempCurveModel->index(row, 0)).toDouble());
        Time.append(TempCurveModel->data(TempCurveModel->index(row, 1)).toDouble());
    }
    //Dinge zu Prüfen:
    // Time = 0; Temp != 0
    qDebug()<<TempCurveModel->rowCount();
    for(int i = 0; i < TempCurveModel->rowCount(); i++){
        if(((int)Time.at(i) == 0)&&((int)Temp.at(i) != 0)){

        }
    }
    qDebug()<<TempCurveModel->rowCount();
}

void MainWindow::SetDoorOpen(int Open3Step)
{
    switch(Open3Step){
        case 0: ui->ServoSlider->setValue(10); break;
        case 1: ui->ServoSlider->setValue(50); break;
        default: ui->ServoSlider->setValue(120);
    }
}

double MainWindow::GetDesiredValue(const int timestep)
{
    double Temp1, Temp2;
    double Time1, Time2;
    for (int row = 1; row < TempCurveModel->rowCount(); row++) {
        Time2 = TempCurveModel->data(TempCurveModel->index(row, 1)).toDouble();

        if(Time2 > timestep){
            Time1 =  TempCurveModel->data(TempCurveModel->index(row-1, 1)).toDouble();

            if(Time1 <= timestep){
                Temp1 =  TempCurveModel->data(TempCurveModel->index(row-1, 0)).toDouble();
                Temp2 =  TempCurveModel->data(TempCurveModel->index(row, 0)).toDouble();

                return Temp1 + (Temp2-Temp1)*((timestep-Time1)/(Time2-Time1));
            }
        }
    }
    return -10;
}

double MainWindow::GetCurrentRamp(const int timestep)
{
    double Temp1, Temp2;
    double Time1, Time2;
    for (int row = 1; row < TempCurveModel->rowCount(); row++) {
        Time2 = TempCurveModel->data(TempCurveModel->index(row, 1)).toDouble();
        if(Time2 > timestep){
            Time1 =  TempCurveModel->data(TempCurveModel->index(row-1, 1)).toDouble();
            if(Time1 <= timestep){
                Temp1 =  TempCurveModel->data(TempCurveModel->index(row-1, 0)).toDouble();
                Temp2 =  TempCurveModel->data(TempCurveModel->index(row, 0)).toDouble();

                return (Temp2-Temp1)/(Time2-Time1);
            }
        }
    }
    return 0;
}


void MainWindow::on_HeaterOffButton_clicked()
{
    ui->HeaterSlider->setValue(0);
}

void MainWindow::on_HeaterSlider_valueChanged(int value)
{
    qDebug()<<"Heater NewVal: "<<value<<"String: "<< QString("AC %1").arg(value);
    ui->HeaterLCD->display(value);
    ui->HeaterLED->turnOn(ui->HeaterLCD->value() != 0);
    ComModul.SendLine(QString("AC=%1").arg(value));
}

void MainWindow::on_HeaterOnButton_clicked()
{
    ui->HeaterSlider->setValue(ui->HeaterSlider->maximum());
}

void MainWindow::on_ServoSlider_valueChanged(int value)
{
    ui->ServoLCD->display(value);
    ComModul.SendLine(QString("Servo=%1").arg(value));
}

void MainWindow::on_CloseButton_clicked()
{
    ui->ServoSlider->setValue(10);
}

void MainWindow::on_OpenButton_clicked()
{
    ui->ServoSlider->setValue(120);
}
