#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

#include <QStandardItem>
#include <QStandardItemModel>
#include "qcustomplot.h"
#include <QTimer>
#include <QtSerialPort>
#include <QLCDNumber>
#include <QCheckBox>

#include "terminalcommunication.h"

#define TEMP_CHAN_COUNT 6


struct TempChannel {
    QLCDNumber * LCD;
    QCheckBox * CheckBoxShow;
    QCPGraph * Graph;
    QVector<double> Values;
};



namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();


private slots:

    void TempCurveItemChanged(QStandardItem * item);
    void TempChanCheckBoxChanged();

    void ParseReceivedLine(QString ReceivedLine);


    void on_ButtonSaveProfile_clicked();

    void on_ButtonLoadProfile_clicked();

    void on_ButtonStart_clicked();

    void on_ButtonCSVexport_clicked();

    void on_HeaterOffButton_clicked();

    void on_HeaterSlider_valueChanged(int value);

    void on_HeaterOnButton_clicked();

    void on_ServoSlider_valueChanged(int value);

    void Tick1000ms();

    void on_CloseButton_clicked();

    void on_OpenButton_clicked();

private:
    Ui::MainWindow *ui;

    bool ConnectToSerialDevice(int VendorID, int ProductID);
    void DisconnectSerialDevice();
    void SerialSendStringWithNLCR(QString String);

    void TempCurveSanityCheck();

    void SetDoorOpen(int Open3Step);

    QStandardItemModel * TempCurveModel;
    QCustomPlot * customPlot;
    QCPGraph * DesiredValueGraph;

    QVector<double> TimeAxisPoints;
    bool IsRunning;

    double GetDesiredValue(const int timestep);
    double GetCurrentRamp(const int timestep);



    TerminalCommunication  ComModul;
    QTimer Tick1000msTimer;

    TempChannel TempChannels[TEMP_CHAN_COUNT];
    void ConfigUI();
};

#endif // MAINWINDOW_H
