#include "terminalcommunication.h"
#include <QDebug>

TerminalCommunication::TerminalCommunication()
{
    SerialPort = 0;
    PortInfo = 0;
    connect(&ReceiveTimeoutTimer,SIGNAL(timeout()),this,SLOT(receiveTimeout()));
    ReceiveTimeoutTimer.setSingleShot(true);
    ReceiveTimeoutTimer.setInterval(300);
    ReceiveBuffer.clear();
}

TerminalCommunication::TerminalCommunication(int VendorID, int ProductID)
{
    TerminalCommunication();
    ConnectToSerialDevice(VendorID,ProductID);
}

TerminalCommunication::~TerminalCommunication()
{
    DisconnectSerialDevice();
}


void TerminalCommunication::SendLine(QString String)
{
    if(isOpen()){
        QByteArray GetTempStrArray;
        GetTempStrArray.append(String);
        GetTempStrArray.append('\r');
        GetTempStrArray.append('\n');
        SerialPort->write(GetTempStrArray,GetTempStrArray.size());
    }
}

void TerminalCommunication::SendByteArray(const QByteArray Array)
{
    if(isOpen()){
        SerialPort->write(Array,Array.size());
    }
}


void TerminalCommunication::receiveTimeout()
{
    ReceiveBuffer.clear();
}



void TerminalCommunication::HandleError(QSerialPort::SerialPortError error)
{
    if(error != QSerialPort::NoError){
        qDebug()<<"QSerial-Error:"<<error<<SerialPort->errorString();
    }
}

void TerminalCommunication::readData()
{
    ReceiveTimeoutTimer.start();
    char TempRead;
    while(SerialPort->getChar(&TempRead)){

        if((TempRead=='\n')||(TempRead=='\r')){
            if(ReceiveBuffer.size() != 0){
               //ReceiveBuffer.append('\0');
               emit LineReceived(QString(ReceiveBuffer));
            }
            ReceiveBuffer.clear();
        }else{
            ReceiveBuffer.append(TempRead);
        }
    }
    ReceiveTimeoutTimer.start();
}


bool TerminalCommunication::ConnectToSerialDevice(int VendorID, int ProductID)
{
    DisconnectSerialDevice();
    foreach (const QSerialPortInfo &info, QSerialPortInfo::availablePorts()) {
        if((info.vendorIdentifier() == VendorID)&&(info.productIdentifier() == ProductID)){
            SerialPort = new QSerialPort(this);
            connect(SerialPort, SIGNAL(readyRead()), this, SLOT(readData()));
            connect(SerialPort, SIGNAL(error(QSerialPort::SerialPortError)), this, SLOT(HandleError(QSerialPort::SerialPortError)));
            SerialPort->setPortName(info.portName());
            if (SerialPort->open(QIODevice::ReadWrite)) {
                PortInfo  = new QSerialPortInfo(*SerialPort);
                qDebug()<<"Connected";
                if (SerialPort->setBaudRate(9600)
                                && SerialPort->setDataBits(QSerialPort::Data8)
                                && SerialPort->setParity(QSerialPort::NoParity)
                                && SerialPort->setStopBits(QSerialPort::OneStop)
                                && SerialPort->setFlowControl(QSerialPort::NoFlowControl)
                                && SerialPort->setDataTerminalReady(true)){
                    return true;
                }else{
                    SerialPort->close();
                    qDebug()<<"Fehler beim Einstellen des Ports";
                    return false;
                }
            }else{
                qDebug()<<"Konnte Port nicht ffnen";
                return false;
            }
        }
    }
    qDebug()<<"Gert nicht gefunden";
    return false;
}


void TerminalCommunication::DisconnectSerialDevice()
{
    if(PortInfo){
       delete PortInfo;
       PortInfo = 0;
    }
    if(SerialPort){
        if(SerialPort->isOpen()){
            SerialPort->close();
        }
        delete SerialPort;
        SerialPort = 0;
    }
}


bool TerminalCommunication::isOpen()
{
    if(PortInfo){
        if(PortInfo->isValid() == false){
            return false;
        }
    }
    if(SerialPort){        
        if(SerialPort->isOpen()){
            return true;
        }
    }
    return false;
}


void TerminalCommunication::setReceiveTimeout(int TimeoutInMs)
{
    ReceiveTimeoutTimer.setInterval(TimeoutInMs);
}

void TerminalCommunication::Flush()
{
    if(isOpen()){
        SerialPort->flush(); //Alles im Buffer raus schreiben
    }
}

void TerminalCommunication::ListPorts()
{
    qDebug()<<"Available Com Ports";
    foreach (const QSerialPortInfo &info, QSerialPortInfo::availablePorts()) {
        QString s =   QObject::tr("Port: ") + info.portName() + "\n"
                    + QObject::tr("Location: ") + info.systemLocation() + "\n"
                    + QObject::tr("Description: ") + info.description() + "\n"
                    + QObject::tr("Manufacturer: ") + info.manufacturer() + "\n"
                    + QObject::tr("Vendor Identifier: ") + (info.hasVendorIdentifier() ? QString("0x")+ QString::number(info.vendorIdentifier(), 16) : QString()) + "\n"
                    + QObject::tr("Product Identifier: ") + (info.hasProductIdentifier() ? QString("0x")+ QString::number(info.productIdentifier(), 16) : QString()) + "\n"
                    + QObject::tr("Busy: ") + (info.isBusy() ? QObject::tr("Yes") : QObject::tr("No")) + "\n";
        qDebug()<<s;
    }
}
