/*
 *  Author: Philipp
 */ 

#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "TempMeasurement.h"
#include "SerialCommunication.h"

#define SERVO_INIT_VAL 10

uint8_t ServoVal = SERVO_INIT_VAL;

uint8_t AC_PWM_CNT = 0;
uint8_t AC_PWM_VAL = 0;

uint8_t AC_Pattern[9] = {	0b00000000,
							0b10000000,
							0b10001000,
							0b10010010,
							0b10101010,
							0b11011010,
							0b11101110,
							0b11111110,
							0b11111111};

uint8_t ACDelayTimer = 0;

void SetServo(void);

inline void HeaterOn(){PORTB &= ~(1<<PINB7);}
inline void HeaterOff(){PORTB |= (1<<PINB7);}


volatile uint16_t ConnectionTimeout = 0;

ISR(TIMER0_COMPA_vect) //1ms Timer
{	
	static uint8_t ServorDelayTimer = 0;
	static uint8_t ACDelayTimer = 20;
	
	if(ConnectionTimeout < 2000){
		ConnectionTimeout++;
	}else{
		AC_PWM_VAL = 0;
		HeaterOff();
	}
	
	if(!(--ACDelayTimer)){
		ACDelayTimer = 20;
		
		if(AC_PWM_VAL >= 8){
			HeaterOn();
			}else{
			if(AC_Pattern[AC_PWM_VAL] & (1<<AC_PWM_CNT)){
				HeaterOn();
				}else{
				HeaterOff();
			}
		}
		AC_PWM_CNT = (AC_PWM_CNT + 1) & 0x7; //0..7			
	}
	if(++ServorDelayTimer >= 10){
		SetServo();
		ServorDelayTimer = 0;
	}
	
}

void Init1MStimer()
{
    OCR0A = 250;          //Alle 1000 s ein interrupt
    TCCR0A |= (1<<WGM01);   //CTC-Modus (Clear Timer on Compare Match)
    TCCR0B |= ((1<<CS01)|(1<<CS00));    //Prescaler 64  250-kHz-Taktung
    TIMSK0 |= (1<<OCIE0A);  //Output-Compare Interrupt enable
}


void InitServoOut()
{
	OCR1A = 40000;          //20 mS Periode	
	TCCR1A |= ((1<<COM1B1)|(1<<WGM11)|(1<<WGM10));   //PWM an Pin OC1B
	TCCR1B |= ((1<<CS11)|(1<<WGM13)|(1<<WGM12));    //Prescaler 8  1MHz-Taktung; CTC-Modus

	OCR1B = 3000;          //Alle 1500 s ein interrupt
	DDRB |= (1<<6);
}


void SetServo(){
	static uint8_t ServoDelay = SERVO_INIT_VAL;
	
	if(ServoDelay < ServoVal){ServoDelay++;}
	if(ServoDelay > ServoVal){ServoDelay--;}
	 
	OCR1B = 952 + (((uint16_t)ServoDelay)<<4);
}


void StartBootloader()
{
		#if defined(__AVR_ATmega32U4__)
		asm volatile("jmp 0x7000"); //0x3800 WordAddress
		#elif defined(_AVR_ATmega16U4__)
		asm volatile("jmp 0x3000"); ; //0x1800 WordAddress
		#else
		#error Unkown AVR-Chip :: Unbekannter AVR
		#endif
}


void CleanRegisterAndStartBootloader()
{	
	send_pstr(PSTR(" - Starte Bootloader"));
	
	_delay_ms(100);
	
	DDRB = 0;
	PORTB = 0;
	OCR1A = 0;
	OCR1B = 0;
	TCCR1A = 0;
	TCCR1B = 0;
	
	OCR0A = 0;
	TCCR0A = 0;
	TCCR0B = 0;
	TIMSK0 = 0;
	
	UDCON = 0x01;
	UDIEN = 0;
	PLLCSR = 0;
	USBCON = (1<<FRZCLK);
	UHWCON = 0;
	
	cli();
	
	_delay_ms(100);
	
	StartBootloader();
}


void CallBootloaOnPinstate()
{
	PORTB |= (1<<2); //Pullup an;
	_delay_us(10);
	if(!(PINB & (1<<2))){
		PORTB = 0;
		StartBootloader();
	}
	PORTB = 0;
}


int main(void)
{
	CallBootloaOnPinstate();
	InitSerialCom();
	InitTempMeasurement();
	InitServoOut();	
	Init1MStimer();
	PORTB |= (1<<7);
	DDRB |= (1<<7);
	sei();

    while(1)
    {
//		DoTriacOut();
		DoTempMeasurement();
        if(DoSerialComReconnecting()){ //Nicht (mehr) verbunden
			ConnectionTimeout = 0;
		}		 
    }
}