/*
 * SerialCommunication.c
 *
 * Created: 05.12.2013 00:44:46
 *  Author: Philipp
 */ 

#include <stdint.h>
#include <avr/pgmspace.h>

#include <avr/io.h>

#include <avr/wdt.h>
#include <avr/interrupt.h>



#include "usb_serial.h"
#include "SerialCommunication.h"
#include "a2i_and_i2a_funcs.h"
#include "TempMeasurement.h"

#define CPU_PRESCALE(n) (CLKPR = 0x80, CLKPR = (n));

#define CMD_BUF_SIZE 64
#define KEYWORD_LENGTH 16

void ParseAndExecuteCommand(const uint8_t * CommandStr);
void send_pstr(const char *s);
void send_str(const uint8_t *s);


extern uint8_t AC_PWM_VAL;
extern uint8_t ServoVal;

extern void CleanRegisterAndStartBootloader();


typedef enum{   CMD_FUNC, CMD_FUNC_ARG_STR, CMD_FUNC_ARG_LONG, //Funktion(void) / Funktion(uint8_t* char)
                CMD_BYTE_RW,CMD_BYTE_R,
                CMD_WORD_RW,CMD_WORD_R,
                CMD_LONG_RW,CMD_LONG_R
            }CommandType;

typedef struct{   uint8_t name[KEYWORD_LENGTH];
                  void *ptr;
                  CommandType Type;
              }CommandEntry;


void TerminalNewLine(){send_pstr(PSTR("\r\n"));}


void PrintADCval(int32_t Chan)
{
    uint8_t count;
    uint8_t ValStringBuffer[12];
	count =  i2a(ValStringBuffer, (int32_t)ADC_Vals[(uint8_t)Chan & 0x07]);
	
	ValStringBuffer[count] = 0;
    ValStringBuffer[11] = 0;
    usb_serial_putchar(' ');
    send_str(ValStringBuffer);		
}

void PrintResval(int32_t Chan)
{
	if(Chan >= CHAN_COUNT-2){return;}
    uint8_t count;
    uint8_t ValStringBuffer[12];
	count =  i2a(ValStringBuffer, (int32_t)ResistorVals[(uint8_t)Chan]);
	
	ValStringBuffer[count] = 0;
    ValStringBuffer[11] = 0;
    usb_serial_putchar(' ');
    send_str(ValStringBuffer);
	usb_serial_putchar('R');
}

void PrintTempVal(int32_t Chan)
{
	if(Chan >= CHAN_COUNT-2){return;}
	uint8_t count;
	uint8_t ValStringBuffer[12];
	count =  i2a(ValStringBuffer, (int32_t)(TempVals[(uint8_t)Chan]));
	
	//fhrende 0 einfgen bei -9,-8...,-1,0,1,2,..,9
	if((count == 1) || ((count == 2) && (ValStringBuffer[0]=='-'))){
		ValStringBuffer[count]	= ValStringBuffer[count-1];
		ValStringBuffer[count-1] = '0';
		count++;
	}
	
	//Komma setzten
	ValStringBuffer[count]	= ValStringBuffer[count-1];
	ValStringBuffer[count-1] = ',';
	ValStringBuffer[count+1] = 0;
	
	ValStringBuffer[11] = 0;
	usb_serial_putchar(' ');
	send_str(ValStringBuffer);
	usb_serial_putchar('*');
	usb_serial_putchar('C');
}


void PrintTemp(int32_t Chan)
{
	usb_serial_putchar(' ');
	PrintTempVal(Chan);
	usb_serial_putchar('C');
}


void PrintAllTemps()
{
	TerminalNewLine();
	for(uint8_t i = 0; i < CHAN_COUNT-2; i++){		
		usb_serial_putchar('T');
		usb_serial_putchar(i+1+'0');
		usb_serial_putchar('=');
		if(TempVals[i] >= TEMP_MAX){			
			send_pstr(PSTR("--"));
		}else{
			PrintTempVal(i);
		}		
		TerminalNewLine();		
	}
}

void PrintHState()
{
	TerminalNewLine();
	if(PINB & (1<<7)){send_pstr(PSTR("PinRead High\r\n"));}else{send_pstr(PSTR("PinRead Low\r\n"));}		
	if(PORTB & (1<<7)){send_pstr(PSTR("PortSet High\r\n"));}else{send_pstr(PSTR("PortSet Low\r\n"));}	
}


const CommandEntry CommandsList[] = {
	
	{"alltemps", PrintAllTemps, CMD_FUNC},
	{"AC", &AC_PWM_VAL, CMD_BYTE_RW},
	{"Servo", &ServoVal, CMD_BYTE_RW},		
	{"GetHState", PrintHState, CMD_FUNC},
		
	{"adc", PrintADCval, CMD_FUNC_ARG_LONG},
	{"res", PrintResval, CMD_FUNC_ARG_LONG},
	{"temp", PrintTempVal, CMD_FUNC_ARG_LONG},
		
	{"bootl", CleanRegisterAndStartBootloader, CMD_FUNC},	
};

const uint8_t CommandsCount = sizeof(CommandsList)/sizeof(CommandEntry);



void InitSerialCom()
{
	CPU_PRESCALE(0);
	usb_init();	
}


// Send a string to the USB serial port.  The string must be in
// flash memory, using PSTR
void send_pstr(const char *s)
{
	uint8_t c;
	while (1) {
		c = pgm_read_byte(s++);
		if (!c) break;
		usb_serial_putchar(c);
	}
}


void send_str(const uint8_t *s)
{
	while (*s) {
		usb_serial_putchar(*s++);
	}
}



uint8_t DoSerialComReconnecting()
{
	uint8_t static IsConnected = 0;	
    
    if (!usb_configured() ||  !(usb_serial_get_control() & USB_SERIAL_DTR)) {
       IsConnected = 0; 
       return 0;
    }else if(IsConnected == 0){
        usb_serial_flush_input();
		IsConnected = 1;			
		send_pstr(PSTR("\r\nDNW Hardware am Start\r\nBuild: Date="));
		send_pstr(PSTR(__DATE__));
		send_pstr(PSTR(" Time="));
		send_pstr(PSTR(__TIME__));
		TerminalNewLine();
    }    
    return DoSerialCom();
}


uint8_t DoSerialCom()
{
    static uint8_t CommandBuffer[CMD_BUF_SIZE];
    static uint8_t CommandLen =  0;
    int16_t Read;
	uint8_t ReceivedSomething = 0;
		
    while((Read = usb_serial_getchar()) != -1){
        if (Read == '\r' || Read == '\n'){//Befehl Ausfhren
            if(CommandLen != 0){
                CommandBuffer[CommandLen] = 0;
                ParseAndExecuteCommand(CommandBuffer);
                CommandLen = 0;
				ReceivedSomething = 1;
            }
        }else if(Read == '\b' || Read == 0x7F){//Backspace
			if(CommandLen){CommandLen--;}
			usb_serial_putchar((uint8_t)Read);
        }else if((Read >= ' ') && (Read <= '~')){//Es handelt sich um ein darstellbares Zeichen
            usb_serial_putchar((uint8_t)Read);
            if(CommandLen >= CMD_BUF_SIZE-1){//Letztes Zeichen;
                CommandLen = 0;
                send_pstr(PSTR("  ERROR - command to long!"));
                TerminalNewLine();
			}else{
				//Alles in Lowercase. ggf. Auskommentieren wenn ungewnscht
				//if((Read >= 'A')&&(Read <= 'Z')){Read += 0x20;}
				
	            CommandBuffer[CommandLen] = Read;
                CommandLen++;
            }            
        }			  				
    }
	return ReceivedSomething;
}



//Sucht das Empfangene Kommando aus der Liste. Und setzt den Lesezeiger im Eingabestring auf die Position nach dem Schlsselwort
//Wird das Kommando nicht gefunden wird -1 zurckgegeben. Der Lesezeige wird dann nicht verndert.
static int8_t GetCommandIndexAndAdvanceCommandStrPtr(const uint8_t ** CommandStrPtr)
{
    uint8_t CmdIdx;
    uint8_t equal;
    const uint8_t * KeyWord;
    const uint8_t * Input;
    for (CmdIdx = 0; CmdIdx < CommandsCount; CmdIdx++)  {

        Input = *CommandStrPtr;
        KeyWord = &(CommandsList[CmdIdx].name[0]);
        equal = 1;

        while (*KeyWord){
            if (*Input++ != *KeyWord++){equal = 0; break;}
        }
        //Prfen ob Befehlswort zu Ende ist (es muss ein Kommando- oder Leerzeichen folgen)
        if(*Input != 0 && *Input != ' ' && *Input != '=' && *Input != '?' ){equal = 0;}

        if(equal){
            *CommandStrPtr = Input;
            return CmdIdx;
        }
    }
    return -1;
}


static void ProcessWriteCommand(const uint8_t CmdIdx, const uint8_t * ValStrPtr)
{
    switch(CommandsList[CmdIdx].Type){
        case CMD_BYTE_RW: *(uint8_t *)CommandsList[CmdIdx].ptr = (uint8_t)a2i(ValStrPtr); break;
        case CMD_WORD_RW: *(uint16_t *)CommandsList[CmdIdx].ptr = (uint16_t)a2i(ValStrPtr); break;
        case CMD_LONG_RW: *(uint32_t *)CommandsList[CmdIdx].ptr =        a2i(ValStrPtr); break;

        default: send_pstr(PSTR("  ERROR - READ ONLY PARAMETER")); return;
    }
    send_pstr(PSTR("  OK"));
}


static void ProcessReadCommand(const uint8_t CmdIdx)
{
    uint8_t count;
    uint8_t ValStringBuffer[12];
    
    switch(CommandsList[CmdIdx].Type){
        case CMD_BYTE_RW: //kein break
        case CMD_BYTE_R:
            count = i2a(ValStringBuffer, (int32_t)(*(uint8_t *)CommandsList[CmdIdx].ptr)); break;

        case CMD_WORD_RW:
        case CMD_WORD_R:
            count =  i2a(ValStringBuffer, (int32_t)(*(uint16_t *)CommandsList[CmdIdx].ptr)); break;

        case CMD_LONG_RW:
        case CMD_LONG_R:
            count = i2a(ValStringBuffer, (int32_t)(*(uint32_t *)CommandsList[CmdIdx].ptr)); break;

        default: return;
    }
    ValStringBuffer[count] = 0;
    ValStringBuffer[11] = 0;
    usb_serial_putchar(' ');
    send_str(ValStringBuffer);
}


static void ProcessReadOrWriteCommand(const uint8_t CmdIdx, const uint8_t * InputStr)
{
    typedef enum{REQ_NONE, REQ_READ, REQ_WRITE}RequestType;
    RequestType Request = REQ_NONE;

    while(*InputStr){
        if(*InputStr == '='){Request = REQ_WRITE; break;}
        if(*InputStr == '?'){Request = REQ_READ; break;}
        if(*InputStr != ' '){break;} //Unzulssiges Zeichen (Leerzeichen werden ignoriert)
        InputStr++;
    }
    InputStr++; //Input zeigt nun auf Stelle nach dem Operatorzeichen

    switch(Request){
       case REQ_READ:  ProcessReadCommand(CmdIdx); break;
       case REQ_WRITE: ProcessWriteCommand(CmdIdx, InputStr); break;
       default:        send_pstr(PSTR("  ERROR - UNKOWN OPERATION")); break;
    }
}



void ParseAndExecuteCommand(const uint8_t * CommandStr)
{
    int8_t CmdIdx = GetCommandIndexAndAdvanceCommandStrPtr(&CommandStr);

    if(CmdIdx == -1){
        send_pstr(PSTR("  ERROR - UNKOWN COMMAND"));
    }else{
        switch(CommandsList[CmdIdx].Type){
            case CMD_FUNC:
                ((void (*)(void))(CommandsList[CmdIdx].ptr))(); break;

            case CMD_FUNC_ARG_STR:
                ((void (*)(const uint8_t*))(CommandsList[CmdIdx].ptr))(CommandStr); break;

            case CMD_FUNC_ARG_LONG:
			     ((void (*)(int32_t))(CommandsList[CmdIdx].ptr))(a2i(CommandStr)); break;		
			
			default:
                ProcessReadOrWriteCommand(CmdIdx, CommandStr); break;
        }
    }
    TerminalNewLine();
}