/*
 * TempMeasurment.c
 *
 * Created: 06.12.2013 16:49:41
 *  Author: Philipp
 */ 



#define OS_BITS 5
#define OS_SAMPLES_COUNT 1024

#include "TempMeasurement.h"
#include <avr/io.h>

#define F_CPU 16000000

#include <util/delay.h>
#include <stdint.h>

uint16_t ADC_Vals[CHAN_COUNT];

uint16_t ResistorVals[CHAN_COUNT-2];
int16_t TempVals[CHAN_COUNT-2];

uint16_t DeltaRefADC = 0;

//Umschaltzeit der Kanle (bis Spannung stabil) < 5 s 


typedef struct {int16_t T; uint16_t R;}TempRefPoint;
	
TempRefPoint RefPoints[]={//Temperaturen in 0,1 C - Widerstnde in 0,1 Ohm
							{-500, 8031},
							{   0,10000},
							{ 500,11940},
							{1000,13851},
							{1500,15733},
							{2000,17586},
							{2500,19410},
							{3000,21205},
							{3500,22972},
							{4000,24710}};
uint8_t	RefPointsCount = sizeof(RefPoints)/sizeof(RefPoints[0]);



void SetMux(uint8_t Channel)
{
	if(Channel & (1<<2)){PORTF |= (1<<1);} else {PORTF &= ~(1<<1);}
	if(Channel & (1<<1)){PORTF |= (1<<4);} else {PORTF &= ~(1<<4);}
	if(Channel & (1<<0)){PORTF |= (1<<5);} else {PORTF &= ~(1<<5);}
}


void InitADC()
{
	DIDR0 |= (1<<0);
	ADMUX  = ((1<<REFS1)|(1<<REFS0)); // 2,56V Referenz, Kanal 0
	ADCSRA = ((1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)); //Prescaler 128: CLK = 16MHz/128 = 125 kHz = 9615 Sp/s 
	ADCSRA |= (1<<ADSC); // Start Conversion
	while(ADCSRA &(1<<ADSC)); //Erste Wandlung Abwarten und verwerfen;
	ADCSRA |= (1<<ADSC); // Start Conversion
}


void InitTempMeasurement()
{ 
	MCUCR = (1<<JTD);
	MCUCR = (1<<JTD);	//Zwei mal um JTAG abzuschlaten (ermglicht PortF als IO)
	DDRF |= ((1<<1)|(1<<4)|(1<<5));
	//DDRF |= (1<<4);	
	SetMux(0);
    _delay_us(5);
	InitADC();
}


void CalcDeltaRefR()
{
	DeltaRefADC = ADC_Vals[1] - ADC_Vals[0];
}

#define R_REF0_IN01 (10000)
#define R_REF1_IN01 (20000)


int16_t CalcTempIn01Celsius(uint16_t MeasuredR)
{
	uint8_t i;
	int16_t RDeltaRef;
	int32_t TempResult;
	if(RefPoints[0].R > MeasuredR){return TEMP_MIN;}
	if(RefPoints[RefPointsCount-1].R < MeasuredR){return TEMP_MAX;}
		
	for(i = 1;i<RefPointsCount; i++){
		if(RefPoints[i].R >= MeasuredR){break;}
	}
	//		DeltaR_Measured * DeltaT_Ref
	//      ---------------------------- + T_Ref_Low
	//			    DeltaR_Ref
	
	RDeltaRef = RefPoints[i].R - RefPoints[i-1].R;
	
	TempResult= ((int32_t)(MeasuredR-RefPoints[i-1].R))*((int32_t)(RefPoints[i].T - RefPoints[i-1].T)); //
	TempResult = (TempResult+ (RDeltaRef>>1))/RDeltaRef;
	
	return (int16_t)TempResult + RefPoints[i-1].T;			
}


uint16_t CalcResistorIn0_1Ohm(uint16_t ADCRawValue)
{
	if(DeltaRefADC == 0){return 0;}

	uint32_t DeltaR;
	uint8_t IsNeg = 0;
	int16_t DeltaADC = ADCRawValue - ADC_Vals[0];
	
	if(DeltaADC == 0){return R_REF0_IN01;}
	
	if(DeltaADC < 0){
		DeltaADC = -DeltaADC;
		IsNeg = 1;
	}	
	DeltaR = ( ((uint32_t)DeltaADC) * (R_REF1_IN01-R_REF0_IN01)) / (uint32_t)DeltaRefADC;
    
	if(IsNeg){
	    if(DeltaR>R_REF0_IN01){
		    return 0;
		}else{
	        return (uint16_t)R_REF0_IN01 - DeltaR;
		}		
	}		
	return (uint16_t)R_REF0_IN01 + DeltaR;
}


void DoTempMeasurement()
{	
    static uint8_t CurrentChannel = 0;
    static uint16_t SampleCount = 0;
	static uint32_t ReadVal = 0;
		
	if(ADCSRA &(1<<ADSC)){ // Messung luft
	    return;	
	}
	
    ReadVal += ADC;
	SampleCount++;
	
	if(SampleCount >= OS_SAMPLES_COUNT){		
		SetMux((CurrentChannel+1) & 0x07); //Frh Umschalten um Zeit zum Einschwingen zu geben;
		
		ADC_Vals[CurrentChannel] = (uint16_t)((ReadVal + (1<<(OS_BITS-1))) >> OS_BITS);				
		
		if(CurrentChannel == 1){
		    CalcDeltaRefR();
		}else if(CurrentChannel > 1){
			ResistorVals[CurrentChannel-2] = CalcResistorIn0_1Ohm(ADC_Vals[CurrentChannel]);
			TempVals[CurrentChannel-2] = CalcTempIn01Celsius(ResistorVals[CurrentChannel-2]);
		}
		CurrentChannel = (CurrentChannel+1) & 0x07;
		
		ReadVal = 0;
		SampleCount = 0;
		_delay_us(3);		
	}
	ADCSRA |= (1<<ADSC); // Start Conversion	
}