
#include <avr/io.h>

/***********************************************************************
* Integer nach ASCII
* Rückgabewert: Anzahl Zeichen
************************************************************************/
uint8_t i2a(uint8_t *dest, int32_t value)
{
	uint8_t i;
	uint8_t *ptr;
	uint8_t bcd[10];
	int32_t rest;
	uint8_t chars;

	rest = value;					// Zahl => BCD
	if (value < 0) rest = -rest;	// Betrag

	for(i=0;rest;i++){
		bcd[i] = rest % 10;			// Rest nach Teilung durch 10
		rest /= 10;
	}
	if (!i){						// falls value==0
		bcd[0] = 0;
		i++;
	}
	chars = i;						// Anzahl Stellen

	ptr = dest;						// BCD => ASCII
	if (value < 0){					// Vorzeichen
		*ptr++ = '-';
		chars++;					// mit Vorzeichen eine Stelle mehr
	}
	do {
		i--;
		*ptr++ = bcd[i] + '0';
	} while (i);

	return chars;
}

/***********************************************************************
* ASCII nach Integer: ignoriert nicht-Zahl zu Beginn
* Rückgabewert: Wert
************************************************************************/
int32_t a2i(const uint8_t *src)
{
	int32_t value;
	uint8_t i;
	uint8_t is_num, neg, value_good;

	value = 0;
	neg = 0;
	value_good = 0;
	for (i=0;i<32;i++){
		is_num = (src[i] >= '0') && (src[i] <= '9');	// Zahl oder keine Zahl
		if ((value_good) && !(is_num)) break;			// Ende Zahl in src erreicht
		if (src[i] == '-') neg = 1;						// negative Zahl ?
		value *= 10;									// dez-shift
		if (is_num){
			value += (int32_t)(src[i]-'0');
			value_good = 1;
		}
	}
	return (neg ? -value : value);
}


uint8_t HexCharToNibble(uint8_t Char)
{
	if((Char>='0')&&(Char<='9')){
		return Char-'0';
	}
	Char |= (1<<5);//Alles lowercase
	if((Char >= 'a') && (Char <='f')){
		return Char -'a' + 10;
	}
	return 0;
}

uint8_t HexToByte(uint8_t HighNibble, uint8_t LowNibble)
{
	return (HexCharToNibble(HighNibble)<<4) | HexCharToNibble(LowNibble);
}

inline uint8_t NibbleToHex(uint8_t Nibble)
{
	Nibble &= 0x0F;
	if(Nibble >= 10){return Nibble - 10 +'A';}
	return Nibble+'0';
}